unit UntRepository;

interface

uses
  SysUtils,Classes;

type
  TDependency=Class;

  TDebFile=Class(TObject)
  private
    fPackageName: String;
    fVersion: String;
    fSource: String;
    fDependency: String;
    fDependencies:TList;
    fInRepository: Boolean;
    function GetDependency(index: Integer): TDependency;
    function GetVersion: String;
  protected
  public
    Constructor Create(const aPackageName:String);
    destructor destroy;override;
    function UpdateVersion(const aVersion:String;const aSource:String):Boolean;
    procedure ClearDependencies;
    function AddDependency(const aDebName:String;const aDeb:TDebFile;const aDebIndex:Integer):TDependency;
    function DependencyCount:Integer;
    procedure SaveToStream(const aMs:TStream);
    procedure LoadFromStream(Const ms:TStream);
    property PackageName:String read fPackageName;
    property Version:String read GetVersion;
    property Source:String read fSource;
    property Dependency:String read fDependency write fDependency;
    property Dependencies[index:Integer]:TDependency read GetDependency;
    property InRepository:Boolean read fInRepository write fInRepository;
  end;

  TDownloadDeb=Class(TObject)
  private
    FPackageName: String;
    FDebFile: TDebFile;
    procedure SetDebFile(const Value: TDebFile);
    procedure SetPackageName(const Value: String);
  protected
  public
    property DebFile:TDebFile read FDebFile write SetDebFile;
    property PackageName:String read FPackageName write SetPackageName;
  end;

  TDependency=Class(Tobject)
  private
    fAvailable: boolean;
    fPackage: TDebFile;
    fPackageName: String;
    fDebIndex: integer;
  protected
  public
    constructor Create(const aPackageName:String;Const aPackage:TDebfile;const aDebIndex:Integer);
    property Package:TDebFile read fPackage write fPackage;
    property Available:boolean read fAvailable write fAvailable;
    property PackageName:String read fPackageName;
    property DebIndex:integer read fDebIndex;
  end;

  TRepository=Class(TObject)
  private
    fList:TList;
    function GetFile(Index: Integer): TDebFile;
    function GetFileIndexByName(const aName:String):integer;
  protected
  public
    function GetFileByName(const aName:String):TDebFile;
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    function Count:Integer;
    Function AddFile(const aName:String):TDebfile;
    property Files[Index:Integer]:TDebFile read GetFile;
    procedure BuildDependencies;
    procedure SaveToStream(const aMs:TStream);
    procedure SaveToFile(const aFileName:String);
    procedure LoadFromFile(Const aFileName:String);
    procedure LoadFromStream(Const ms:TStream);
  end;

implementation

uses
  UntMisc;

{ TDebFile }

function TDebFile.AddDependency(const aDebName:String;const aDeb: TDebFile;const aDebIndex:Integer): TDependency;
var
  i:Integer;
begin
  for i:=0 to fDependencies.count-1 do
    begin
      if TDependency(fDependencies[i]).PackageName=aDebName then
        begin
          result:=TDependency(fDependencies[i]);
          exit;
        end;
    end;
  result:=TDependency.Create(aDebName,aDeb,aDebIndex);
  fDependencies.Add(result);
end;

procedure TDebFile.ClearDependencies;
begin
  while fDependencies.count>0 do
    begin
      TDependency(fDependencies[0]).Free;
      fDependencies.Delete(0);
    end;
end;

constructor TDebFile.Create(const aPackageName: String);
begin
  fPackageName:=aPackageName;
  fVersion:='';
  fSource:='';
  fDependency:='';
  fDependencies:=TList.Create;
  fInRepository:=False;
end;

function TDebFile.DependencyCount: Integer;
begin
  result:=fDependencies.Count;
end;

destructor TDebFile.destroy;
begin
  ClearDependencies;
  fDependencies.Free;
  inherited;
end;

function TDebFile.GetDependency(index: Integer): TDependency;
begin
  result:=TDependency(fDependencies[index]);
end;

function TDebFile.GetVersion: String;
begin
  if pos(':',fVersion)>0 then
    result:=copy(fversion,pos(':',fVersion)+1,length(fversion))
  else
    result:=fVersion;
end;

procedure TDebFile.LoadFromStream(const ms: TStream);
var
  i,j,k:Integer;
  dep:TDependency;
  b:Boolean;
  s:String;
begin
  ClearDependencies;
  fPackageName:=ReadString(Ms);
  fVersion:=ReadString(Ms);
  fSource:=ReadString(Ms);
  fDependency:=ReadString(Ms);
  fInRepository:=ReadBool(Ms);
  j:=ReadInt(ms);
  for i:=0 to j-1 do
    begin
      k:=ReadInt(ms);
      b:=ReadBool(ms);
      if not b then
        s:=readString(ms)
      else
        s:='';
      dep:=TDependency.Create(s,nil,k);
      dep.Available:=b;
      fDependencies.Add(dep);
    end;
end;

procedure TDebFile.SaveToStream(const aMs: TStream);
var
  i:Integer;
begin
  WriteString(aMs,fPackageName);
  WriteString(aMs,fVersion);
  WriteString(aMs,fSource);
  WriteString(aMs,fDependency);
  WriteBool(aMs,fInRepository);
  WriteInt(aMs,fDependencies.Count);
  for i:=0 to fDependencies.count-1 do
    begin
      WriteInt(aMs,Dependencies[i].DebIndex);
      writeBool(aMs,Dependencies[i].Available);
      if not Dependencies[i].Available then
        writeString(aMs,Dependencies[i].PackageName);
    end;
end;

function TDebFile.UpdateVersion(const aVersion, aSource: String):Boolean;
begin
  result:=False;
  if aVersion>fVersion then
    begin
      fVersion:=aVersion;
      fSource:=aSource;
      result:=True;
    end;
end;

{ TRepository }

function TRepository.AddFile(const aName: String): TDebfile;
begin
  result:=GetFileByName(aName);
  if not Assigned(result) then
    begin
      result:=TDebFile.Create(aName);
      fList.Add(result);
    end;
end;

procedure TRepository.BuildDependencies;
var
  i,k:Integer;
  s,s1:string;
  d:TDebFile;
  p1,p2:Integer;
begin
  for i:=0 to fList.Count-1 do
    begin
      Files[i].ClearDependencies;
      s:=trim(Files[i].Dependency);
      if s<>'' then
        begin
          s:=s+', ';
          while s<>'' do
            begin
              p1:=pos(' ',s);
              p2:=pos(',',s);
              if p1<p2 then
                begin
                  s1:=trim(copy(s,1,p1-1));
                  s:=trim(copy(s,pos(')',s)+3,length(s)-pos(')',s)));
                end
              else
                begin
                  s1:=trim(copy(s,1,p2-1));
                  s:=trim(copy(s,p1+1,length(s)-p1));
                end;
              k:=GetFileIndexByName(s1);
              if k<>-1 then
                d:=Files[k]
              else
                d:=nil;
              Files[i].AddDependency(s1,d,k).Available:=Assigned(d);
            end;
        end;
    end;
end;

procedure TRepository.Clear;
begin
  while fList.count>0 do
    begin
       TDebFile(FList[0]).Free;
       fList.Delete(0);
    end;
end;

function TRepository.Count: Integer;
begin
  result:=fList.Count;
end;

constructor TRepository.Create;
begin
  fList:=TList.Create;
end;

destructor TRepository.Destroy;
begin
  clear;
  fList.Free;
  inherited;
end;

function TRepository.GetFile(Index: Integer): TDebFile;
begin
  result:=TDebfile(fList[Index]);
end;

function TRepository.GetFileByName(const aName: String): TDebFile;
var
  i:Integer;
begin
  result:=nil;
  for i:=0 to fList.count-1 do
    begin
      if UpperCase(Files[i].PackageName)=UpperCase(aName) then
        begin
          result:=Files[i];
          exit;
        end;
    end;
end;

function TRepository.GetFileIndexByName(const aName: String): integer;
var
  i:Integer;
begin
  result:=-1;
  for i:=0 to fList.count-1 do
    begin
      if UpperCase(Files[i].PackageName)=UpperCase(aName) then
        begin
          result:=i;
          exit;
        end;
    end;
end;

procedure TRepository.LoadFromFile(const aFileName: String);
var
  ms:TFileStream;
begin
  ms:=TFileStream.Create(aFileName,fmOpenRead);
  try
    LoadFromStream(ms);
  finally
    ms.Free;
  end;
end;

procedure TRepository.LoadFromStream(const ms: TStream);
var
  j,i:Integer;
  d:TdebFile;
begin
  Clear;
  j:=ReadInt(ms);
  for i:=0 to j-1 do
    begin
      d:=TDebfile.Create('');
      d.LoadFromStream(Ms);
      fList.Add(d);
    end;
  for i:= 0 to fList.Count-1 do
    begin
      for j:=0 to Files[i].DependencyCount-1 do
        begin
          if Files[i].Dependencies[j].Available then
            Files[i].Dependencies[j].Package:= Files[Files[i].Dependencies[j].DebIndex];
        end;
    end;
//  BuildDependencies;
end;

procedure TRepository.SaveToFile(const aFileName: String);
var
  ms:TFileStream;
begin
  ms:=TFileStream.Create(aFileName,fmCreate);
  try
  SaveToStream(ms);
  finally
    ms.Free;
  end;
end;

procedure TRepository.SaveToStream(const aMs: TStream);
var
  i:Integer;
begin
  WriteInt(aMs,fList.Count);
  for i:=0 to fList.Count-1 do
    begin
      //WriteString(aMs,Files[i].PackageName);
      files[i].SaveToStream(aMs);
    end;
end;

{ TDependency }

constructor TDependency.Create(const aPackageName:String;const aPackage: TDebfile;const aDebIndex:Integer);
begin
  fPackageName:=aPackageName;
  fPackage:=aPackage;
  fDebIndex:=aDebIndex;
end;

{ TDownloadDeb }

procedure TDownloadDeb.SetDebFile(const Value: TDebFile);
begin
  FDebFile := Value;
end;

procedure TDownloadDeb.SetPackageName(const Value: String);
begin
  FPackageName := Value;
end;

end.
